import { kebab, setAttribute } from './common';
/**
 * When ObserveAttribute is added to a property in a custom element,
 * any changes made on the attribute (through DOM or setAttribute) are reflected back to the class property
 *
 * When set to true, then any changes done on the property programmatically are reflected on the attribute
 *
 * @param reflectToAttribute when true, reflect property changes to its attribute
 */
export function ObserveAttribute(reflectToAttribute = false) {
    return (target, propertyKey) => {
        const ctor = target.constructor;
        const observedAttrs = ctor.observedAttributes;
        const observedAttr = kebab(propertyKey);
        observedAttrs.push(observedAttr);
        let propertyValue = '';
        Object.defineProperty(target, propertyKey, {
            configurable: true,
            get: function () {
                return propertyValue;
            },
            set: function (value) {
                const self = this;
                propertyValue = value;
                if (self.__attached) {
                    if (reflectToAttribute) {
                        setAttribute(this, propertyKey, value);
                    }
                    self.__render();
                }
            }
        });
    };
}
//# sourceMappingURL=observeAttribute.js.map