import { setProperties, updateProperties } from '../common';
export function render(parent, vnode, previous) {
    return updateElement(parent, vnode, previous);
}
function createNode(node) {
    if (typeof node === 'string') {
        return document.createTextNode(node);
    }
    const element = document.createElement(node.type);
    setProperties(element, node.properties);
    node.children
        .map(createNode)
        .forEach(element.appendChild.bind(element));
    return element;
}
function changed(node1, node2) {
    return typeof node1 !== typeof node2 ||
        typeof node1 === 'string' && node1 !== node2 ||
        node1.type !== node2.type;
}
/**
 * Update custom element template whenever a property changes.
 * @param parent
 * @param newNode
 * @param oldNode
 * @param index Starts at 1 because index 0 is styles
 */
function updateElement(parent, newNode, oldNode, index = 1) {
    if (oldNode == null) {
        parent.appendChild(createNode(newNode));
    }
    else if (newNode == null) {
        parent.removeChild(parent.childNodes[index]);
    }
    else if (changed(newNode, oldNode)) {
        parent.replaceChild(createNode(newNode), parent.childNodes[index]);
    }
    else if (newNode.type) {
        updateProperties(parent.childNodes[index], newNode.properties, oldNode.properties);
        const newLength = newNode.children.length;
        const oldLength = oldNode.children.length;
        for (let i = 0; i < newLength || i < oldLength; i++) {
            updateElement(parent.childNodes[index], newNode.children[i], oldNode.children[i], i);
        }
    }
    return newNode;
}
//# sourceMappingURL=render.js.map